// DisplayName = Retrieval - Team Aerial Combat

//--- GAME RULES BEGIN ---
//Determine location of artifact
//Escort Jericho to artifact
//Drive over artifact with Jericho
//Escort Jericho to base
//Dont touch the ground...it hurts.
//--- GAME RULES END ---


//=====================================================================================
//	TAC2 - Retrieval Mission Code								Wizard_TPG - 11/5/2001
//
//	This code includes:
//				Retrieval Game Code
//				Player spawns with repair pack
//
//=====================================================================================


//exec the AI scripts
exec("scripts/aiCTF.cs");

package TAC_RetrievalGame
{
	function TACArtifact::objectiveInit(%data, %artifact)
	{
	   $ArtifactStatus = "<In the Field>";
	}
};




//-- tracking  ---
// .deaths .kills .genDefends .carrierKills .flagDefends .escortAssists .suicides .teamKills .turretKills .flagCaps
// .flagReturns .genDestroys .genRepairs

function TAC_RetrievalGame::initGameVars(%game)
{
	%game.SCORE_PER_VEHICLE_APC_KILL = 3;  	//enemy team gets this for killing your apc from a vehicle
	%game.SCORE_PER_OTHER_APC_KILL = 1;		//enemy team gets this for killing apc otherwise
	%game.SCORE_PER_OWN_APC_KILL = -3;		//Your team gets this if you tk an apc
	%game.SCORE_PER_BASE_APC_KILL = 0;		//Enemy team gets this if your veihcle is destroyed by enemy base turrets
	%game.SCORE_PER_TURRET_APC_KILL = 1;	//Enemy team gets this for kills by other turrets
	%game.SCORE_PER_CRASH_APC_KILL = -1;	//Your team gets this if you crash an apc
	%game.SCORE_PER_COLLIDE_APC_KILL = 1;	//Your team gets this if you collide with an enemy causing their vehicle to be destroyed
	%game.SCORE_PER_ARTIFACT_CAPTURE = 20000;

	%game.SCORE_PER_SUICIDE = 0;
	%game.SCORE_PER_TEAMKILL = 0;
	%game.SCORE_PER_DEATH = 0;

	%game.SCORE_PER_KILL = 0;
	%game.SCORE_PER_PLYR_FLAG_CAP = 0;
	%game.SCORE_PER_TEAM_FLAG_CAP = 10;
	%game.SCORE_PER_GEN_DESTROY = 0;
	%game.SCORE_PER_ESCORT_ASSIST = 0;

	%game.SCORE_PER_TURRET_KILL = 0;
	%game.SCORE_PER_FLAG_DEFEND = 0;
	%game.SCORE_PER_CARRIER_KILL = 0;
	%game.SCORE_PER_FLAG_RETURN = 0;
	%game.SCORE_PER_GEN_DEFEND = 0;
	%game.SCORE_PER_GEN_REPAIR = 0;

	%game.FLAG_RETURN_DELAY = 15 * 1000;

	%game.TIME_CONSIDERED_FLAGCARRIER_THREAT = 0;  //after damaging enemy flag carrier
	%game.RADIUS_GEN_DEFENSE = 0;  //meters
	%game.RADIUS_FLAG_DEFENSE = 0;  //meters

	%game.fadeTimeMS = 2000;
	%game.notifyMineDist = 7.5;
}

//--------------------------------------------------------------------------------------------------------------------

function TAC_RetrievalGame::missionLoadDone(%game)
{
	%game.numTeams = 2;

   //continue with the default function
   DefaultGame::missionLoadDone(%game);
}

function TAC_RetrievalGame::setUpTeams(%game)
{
	DefaultGame::setUpTeams(%game);

	//Set up sensor group colours
	%bluemask |= (1 << 6);
	setSensorGroupColor(1, %bluemask, "0 0 255 255"); //blue
	setSensorGroupColor(2, %bluemask, "0 0 255 255"); //blue

	//spawn Artifact
	%game.CreateArtifact();

	//Start distance check loop
	%game.schedule(1000,"ArtifactDistanceCheck");

   	//Set to two teams
   	%game.numTeams = 2;
   	setSensorGroupCount(%game.numTeams + 1);
}

function TAC_RetrievalGame::GetArtifactSpawnPoint(%game)
{
	%team = 6;
	if(nameToId(ArtifactSpawn1) != -1)
	{
		%imax = 2;
		for(%i=1;%i<%imax; %i++)
		{
			%nextname = "ArtifactSpawn" @ %i;
			if(nameToId(%nextname) != -1)
			{
				%imax++;
				%spawnPosition[%i] = nameToId(%nextname).position;
			}
		}

		//Pick random point
		%maxspawnpoints = %imax - 1;
		%testvalue = mFloor(getRandom() * %maxspawnpoints);
		if(%testvalue > %maxspawnpoints)
			%testvalue = %maxspawnpoints;
		if(%spawnPosition[%testvalue] $= "")
			%testvalue = 1;
		%spawnLoc = %spawnPosition[%testvalue];
	}
	else
		%spawnLoc = -1;
	if(%spawnLoc == -1)
	{
		echo("TAC_Retrieval: Default Artifact spawn method failure.  Looking for team 6 spawns");
		%spawnLoc = %game.pickTeamSpawn(%team);
	}


	//Determine ground point
	for(%loc = 0;%loc<7;%loc++)
	{
		%locdata[%loc] = getword(%spawnLoc,%loc);
	}
	%start = %locdata[0] @ " " @ %locdata[1] @ " 2000";
	%end = %locdata[0] @ " " @ %locdata[1] @ " -1";
	%ground = containerRayCast(%start, %end, $TypeMasks::TerrainObjectType, 0);
	%z = getWord(%ground, 3) + 0.1;
	for(%loc = 0;%loc<7;%loc++)
	{
		if(%loc == 2)
			%outputSpawn = %outputSpawn SPC %z;
		else if(%loc == 0)
			%outputSpawn = %locdata[0];
		else
			%outputSpawn = %outputSpawn SPC %locdata[%loc];
	}
	return %outputSpawn;
}

function TAC_RetrievalGame::CreateArtifact(%game)
{
	%outputSpawn = %game.GetArtifactSpawnPoint();
	if(%outputSpawn == -1)
		Error("TAC_Retrieval: Artifact did not spawn!");
	%artifact = TACArtifact.create(%team,%outputSpawn);
	$ArtifactStatus	= "<In the Field>";
	%artifact.artifact = true;
	%game.artifact = %artifact;
	setTargetName(%artifact.getTarget(), "Artifact");
}

function TAC_RetrievalGame::RespawnArtifact(%game, %obj, %reason)
{
	%outputSpawn = %game.GetArtifactSpawnPoint();
	if(%outputSpawn == -1)
		Error("TAC_Retrieval: Artifact could not re-spawn!");
	%obj.setTransform(%outputSpawn);
	$ArtifactStatus	= "<In the Field>";
	if(%reason $= "liquid")
		messageAll('MsgArtifactRespawn', 'The artifact has randomly respawned due to being lost in liquid~wfx/misc/Artifact_Respawn.wav', $ArtifactStatus );
	else if(%reason $= "timeout")
		messageAll('MsgArtifactRespawn', 'The artifact has randomly respawned as it was not retrieved within 5 minutes of being dropped.~wfx/misc/Artifact_Respawn.wav', $ArtifactStatus );
	else if(%reason $= "oob")
		messageAll('MsgArtifactRespawn', 'The artifact has randomly respawned as it was taken out of the mission area.~wfx/misc/Artifact_Respawn.wav', $ArtifactStatus );
}

function TAC_RetrievalGame::equip(%game, %player)
{
   for(%i =0; %i<$InventoryHudCount; %i++)
      %player.client.setInventoryHudItem($InventoryHudData[%i, itemDataName], 0, 1);

   %player.setInventory(RepairKit,1);
   %player.setInventory(FlareGrenade,6);
   %player.setInventory(Blaster,1);
   %player.setInventory(Chaingun,1);
   %player.setInventory(ChaingunAmmo, 100);
   %player.setInventory(Disc,1);
   %player.setInventory(DiscAmmo, 20);
   %player.setInventory(RepairPack,1);
   %player.setInventory(TargetingLaser, 1);
   %player.weaponCount = 3;
   %player.use("Blaster");
}

function TAC_RetrievalGame::VehicleKill(%game, %data, %obj)
{
	DefaultGame::VehicleKill(%game, %data, %obj);
}

function TAC_RetrievalGame::timeLimitReached(%game)
{
	echo("LOG: game over (timelimit)");
	%game.gameOver();
   schedule( 5000, %game, cycleMissions );
}

function TAC_RetrievalGame::scoreLimitReached(%game)
{
	echo("LOG: game over (scorelimit)");
	%game.gameOver();
   schedule( 5000, %game, cycleMissions );
}

function TAC_RetrievalGame::gameOver(%game)
{
	//send the winner message
	%winner = "";
	if ($teamScore[1] > $teamScore[2])
		%winner = $teamName[1];
	else if ($teamScore[2] > $teamScore[1])
		%winner = $teamName[2];

	if (%winner $= 'Storm')
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.stowins.wav" );
	else if (%winner $= 'Inferno')
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.infwins.wav" );
	else
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.gameover.wav" );

	//call the default
	DefaultGame::gameOver(%game);

   %game.sendDebriefing();
	messageAll('MsgClearObjHud', "");
	for(%i = 0; %i < ClientGroup.getCount(); %i ++)
	{
		%client = %group.getObject(%i);
		%game.resetScore(%client);
	}
   for(%j = 1; %j <= %game.numTeams; %j++)
		$TeamScore[%j] = 0;
}


////////////////////////////////////////////////////////////////////////////////////////
function TAC_RetrievalGame::clientMissionDropReady(%game, %client)
{
   	messageClient(%client, 'MsgClientReady',"", %game.class);
   	for(%i = 1; %i <= %game.numTeams; %i++)
   	{
      	messageClient(%client, 'MsgRetrievalAddTeam', "", %i, $TeamName[%i], $TeamScore[%i], $ArtifactStatus);
   	}
	messageClient(%client, 'MsgMissionDropInfo', '\c0You are in mission %1 (%2).', $MissionDisplayName, $MissionTypeDisplayName, $ServerName );

	DefaultGame::clientMissionDropReady(%game, %client);
}


function TAC_RetrievalGame::assignClientTeam(%game, %client, %respawn)
{
	DefaultGame::assignClientTeam(%game, %client, %respawn);
	// if player's team is not on top of objective hud, switch lines
	messageClient(%client, 'MsgCheckTeamLines', "", %client.team);
}

function TAC_RetrievalGame::updateKillScores(%game, %clVictim, %clKiller, %damageType, %implement)
{
	if(%clKiller <= 0 || %clKiller $= "")
	{
		%game.awardScoreDeath(%clVictim);
	}
	else if (%game.testKill(%clVictim, %clKiller)) //verify victim was an enemy
	{
		%game.awardScoreKill(%clKiller);
		%game.awardScoreDeath(%clVictim);
	}
	else
	{
		if (%game.testSuicide(%clVictim, %clKiller, %damageType))  //otherwise test for suicide
		{
			%game.awardScoreSuicide(%clVictim);
		}
		else
		{
			if (%game.testTeamKill(%clVictim, %clKiller)) //otherwise test for a teamkill
				%game.awardScoreTeamKill(%clVictim, %clKiller);
		}
	}
}

function TAC_RetrievalGame::checkScoreLimit(%game, %team)
{
	echo("*** current score = ",$TeamScore[1]," - ",$TeamScore[2]);
	%scoreLimit = 10000;
   	if($TeamScore[%team] >= %scoreLimit)
      	%game.scoreLimitReached();
}

function TAC_RetrievalGame::enterMissionArea(%game, %playerData, %player)
{
	if(%player.getState() $= "Dead")
	return;

   	%player.client.outOfBounds = false;
   	messageClient(%player.client, 'EnterMissionArea', '\c1You are back in the mission area.');
	//the instant a player leaves the mission boundary, the flag is dropped, and the return is scheduled...
	if (%player.holdingFlag > 0)
	{
		cancel($FlagReturnTimer[%player.holdingFlag]);
		$FlagReturnTimer[%player.holdingFlag] = "";
	}
}

function TAC_RetrievalGame::leaveMissionArea(%game, %playerData, %player)
{
	if(%player.getState() $= "Dead")
	return;

	// maybe we'll do this just in case
   	%player.client.outOfBounds = true;
	// if the player is holding a flag, strip it and throw it back into the mission area
	// otherwise, just print a message
	if(%player.holdingFlag > 0)
		%game.boundaryLoseFlag(%player);
	else
      messageClient(%player.client, 'MsgLeaveMissionArea', '\c1You have left the mission area.');
	echo("LOG: "@%player@" ("@%player.client@") leave mission area");
}


function TAC_RetrievalGame::PickUpArtifact(%game, %vehicle, %artifact)
{
	cancel(%game.ground);
	$ArtifactStatus = "<Carried On MPB>";
	%vehicle.playThread($DeployThread,"deploy");
	%vehicle.mountObject(%artifact, 1);
	%vehicle.canDeploy = false;
	%pilot = %vehicle.lastpilot.client;
	%enemyteam = %pilot.team == 1 ? 2 : 1;
	messageClient(%pilot, 'msgRetievalArtifactTaken', '\c0You have loaded the artifact onto your MPB! Now get it to the retrieval point!~wfx/misc/Pilot_Artifact_Loaded.wav', $ArtifactStatus);
	messageTeamExcept(%pilot, 'msgRetievalArtifactTaken', '\c0Your team has loaded the Artifact onto your MPB!~wfx/misc/Friend_Artifact_Loaded.wav', $ArtifactStatus);
	messageTeam(%enemyteam, 'msgRetievalArtifactTaken', '\c0Enemy team has loaded the Artifact onto their MPB! Stop Them!~wfx/misc/Enemy_Artifact_Loaded.wav', $ArtifactStatus);
	messageTeam(0, 'msgRetievalArtifactTaken', '\c0%2 team has loaded the Artifact onto their MPB!~wfx/misc/Artifact_Loaded.wav', $ArtifactStatus, $TeamName[%pilot.team]);
	cancel(%game.ArtifactTimerSchedule);
}

function TAC_RetrievalGame::ArtifactTimer(%game)
{
	%artifact = %game.artifact;
	%game.ArtifactTimerSchedule = %game.schedule(300000, "RespawnArtifact", %artifact, "timeout");
}

function TAC_RetrievalGame::GroundArtifact(%game)
{
	%artifact = %game.artifact;
	%spawnLoc = %artifact.getPosition();
	for(%loc = 0;%loc<7;%loc++)
	{
		%locdata[%loc] = getword(%spawnLoc,%loc);
	}
	%start = %locdata[0] @ " " @ %locdata[1] @ " 2000";
	%end = %locdata[0] @ " " @ %locdata[1] @ " -1";
	%ground = containerRayCast(%start, %end, $TypeMasks::TerrainObjectType, 0);
	%z = getWord(%ground, 3) + 0.1;
	for(%loc = 0;%loc<7;%loc++)
	{
		if(%loc == 2)
			%outputSpawn = %outputSpawn SPC %z;
		else if(%loc == 0)
			%outputSpawn = %locdata[0];
		else
			%outputSpawn = %outputSpawn SPC %locdata[%loc];
	}
	%artifact.setTransform( %outputSpawn );
}

function TAC_RetrievalGame::VehicleKill(%game, %data, %obj)
{
	Parent::VehicleKill(%game, %data, %obj);

   	if ( %obj.getDataBlock().className $= WheeledVehicleData && %obj.canDeploy == false)
   	{
		$ArtifactStatus = "<In the Field>";
		messageAll('msgRetievalArtifactLost', '\c0The MPB carrying the Artifact was destroyed!~wfx/misc/MPB_Artifact_Destroyed.wav', $ArtifactStatus);
		%game.ArtifactTimer();

		//move artifact onto ground
		%game.ground = %game.schedule(27000, "GroundArtifact");
	}
}

function TAC_RetrievalGame::ArtifactDistanceCheck(%game)
{
	%artifact = %game.artifact;
	%artifactPosition = %artifact.getTransform();
	%Team1ReturnDist = vectorDist( %artifactPosition, nameToId(Team1Return).position );
	%Team2ReturnDist = vectorDist( %artifactPosition, nameToId(Team2Return).position );

	if(%Team1ReturnDist <= 10 && %Team1ReturnDist > 0)
	{
		%team = 1;
		%teampoints = %game.SCORE_PER_ARTIFACT_CAPTURE;
		$TeamScore[%team] += %teampoints;
		%game.checkScoreLimit(%team);
	}
	else if(%Team2ReturnDist <= 10 && %Team2ReturnDist > 0)
	{
		%team = 2;
		%teampoints = %game.SCORE_PER_ARTIFACT_CAPTURE;
		$TeamScore[%team] += %teampoints;
		%game.checkScoreLimit(%team);
	}
	else
		%game.schedule(1000,"ArtifactDistanceCheck");
}

function TAC_RetrievalGame::DisplaySpecialDebriefMessage(%game, %client, %firstTeam, %secondTeam)
{
	messageClient( %client, 'MsgDebriefResult', "", '<just:center>Team %1 wins because they managed to recover the Artifact!', $TeamName[%firstTeam]);
}

function TAC_RetrievalGame::DisplaySpecialTeamScore(%game, %client, %score, %team)
{
	%score -= %game.SCORE_PER_ARTIFACT_CAPTURE;
	messageClient( %client, 'MsgDebriefAddLine', "", '<tab:5,155><clip:150>\t%1</clip>\t%2 + Artifact Recovered', $TeamName[%team], %score );
}

function TAC_RetrievalGame::leaveMissionArea(%game, %playerData, %player)
{
   	defaultgame::leaveMissionArea(%game, %playerData, %player);

	//check if player is leaving mission area with artifact -- respawn it if they are
	if(!%player.mpbpilot)
		return;
	%vehicle = %player.mpbpilot;
	if(%vehicle.canDeploy)
		return;
	%artifact = %game.artifact;
	%vehicle.unmountObject(%artifact);
	%game.RespawnArtifact(%artifact, "oob");
	%vehicle.setThreadDir($DeployThread, false);
	%vehicle.playThread($DeployThread,"deploy");
	%vehicle.playAudio(0, MobileBaseTurretDeploySound);
	%vehicle.canDeploy = true;
}