// Notes:
// - respawning vehicles with turrets (bomber/tank) will not setup the turret properly

//Damage Rate for entering Liquid
$VehicleDamageLava       = 0.0325;
$VehicleDamageHotLava    = 0.0325;
$VehicleDamageCrustyLava = 0.0325;

$NumVehiclesDeploy = 0;

$VehicleDecalThread = 1;

//**************************************************************
//* GENERAL PURPOSE FUNCTIONS
//**************************************************************

function VehicleData::onAdd(%data, %obj)
{
   Parent::onAdd(%data, %obj);
   if((%data.sensorData !$= "") && (%obj.getTarget() != -1))
      setTargetSensorData(%obj.getTarget(), %data.sensorData);
   %obj.setRechargeRate(%data.rechargeRate);
   // set full energy
   %obj.setEnergyLevel(%data.MaxEnergy);

   if(%obj.disableMove)
      %obj.immobilized = true;
   if(%obj.deployed)
   {
      if($countDownStarted)
         %data.schedule(($Host::WarmupTime * 1000) / 2, "vehicleDeploy", %obj, 0, 1);
      else
      {
         $VehiclesDeploy[$NumVehiclesDeploy] = %obj;
         $NumVehiclesDeploy++;
      }
   }
   if(%obj.mountable || %obj.mountable $= "")
      %data.isMountable(%obj, true);
   else
      %data.isMountable(%obj, false);

   %obj.setSelfPowered();
//   %data.canObserve = true;

	%decal = Game.DefineTeamSkin(%obj.team, "vehicle");
	if(%decal !$= "")
		%obj.schedule(5000, "playThread", $VehicleDecalThread, %decal);
}

function VehicleData::onRemove(%this, %obj)
{
   // if there are passengers/driver, kick them out
   %this.deleteAllMounted(%obj);
   for(%i = 0; %i < %obj.getDatablock().numMountPoints; %i++)
      if (%obj.getMountNodeObject(%i)) {
         %passenger = %obj.getMountNodeObject(%i);
         %passenger.unmount();
      }
   vehicleListRemove(%obj.getDataBlock(), %obj);
   if(%obj.lastPilot.lastVehicle == %obj)
      %obj.lastPilot.lastVehicle = "";

   Parent::onRemove(%this, %obj);
}

function VehicleData::onDamage(%this,%obj)
{
   %damage = %obj.getDamageLevel();
   if (%damage >= %this.destroyedLevel)
   {
      if(%obj.getDamageState() !$= "Destroyed")
      {
         if(%obj.respawnTime !$= "")
            %obj.marker.schedule = %obj.marker.data.schedule(%obj.respawnTime, "respawn", %obj.marker);
         %obj.setDamageState(Destroyed);
      }
   }
   else
   {
      if(%obj.getDamageState() !$= "Enabled")
         %obj.setDamageState(Enabled);
   }
}

function HoverVehicle::useCreateHeight()
{
	//this function is declared to prevent console error msg spam...
}

function WheeledVehicle::useCreateHeight()
{
	//this function is declared to prevent console error msg spam...
}

function AssaultVehicle::onDamage(%this, %obj)
{
   (%obj.getMountNodeObject(10)).setDamagelevel(%obj.getDamageLevel());
   Parent::onDamage(%this, %obj);
}

function BomberFlyer::onDamage(%this, %obj)
{
   (%obj.getMountNodeObject(10)).setDamagelevel(%obj.getDamageLevel());
   Parent::onDamage(%this, %obj);
}

function MobileBaseVehicle::onDamage(%this, %obj)
{
   if(isObject(%obj.getMountNodeObject(1)))
      (%obj.getMountNodeObject(1)).setDamagelevel(%obj.getDamageLevel());
   Parent::onDamage(%this, %obj);
}

function VehicleData::onEnterLiquid(%data, %obj, %coverage, %type)
{
   switch(%type)
   {
      case 0:
         //Water
         %obj.setHeat(0.0);
      case 1:
         //Ocean Water
         %obj.setHeat(0.0);
      case 2:
         //River Water
         %obj.setHeat(0.0);
      case 3:
         //Stagnant Water
         %obj.setHeat(0.0);
      case 4:
         //Lava
         %obj.liquidDamage(%data, $VehicleDamageLava, $DamageType::Lava);
      case 5:
         //Hot Lava
         %obj.liquidDamage(%data, $VehicleDamageHotLava, $DamageType::Lava);
      case 6:
         //Crusty Lava
         %obj.liquidDamage(%data, $VehicleDamageCrustyLava, $DamageType::Lava);
      case 7:
         //Quick Sand
   }
}

function FlyingVehicle::liquidDamage(%obj, %data, %damageAmount, %damageType)
{
   if(%obj.getDamageState() !$= "Destroyed")
   {
      %data.damageObject(%obj, 0, "0 0 0", %damageAmount, %damageType);
      %obj.lDamageSchedule = %obj.schedule(50, "liquidDamage", %data, %damageAmount, %damageType);
      passengerLiquidDamage(%obj, %damageAmount, %damageType);
   }
   else
      %obj.lDamageSchedule = "";
}

function WheeledVehicle::liquidDamage(%obj, %data, %damageAmount, %damageType)
{
   if(%obj.getDamageState() !$= "Destroyed")
   {
      %data.damageObject(%obj, 0, "0 0 0", %damageAmount, %damageType);
      %obj.lDamageSchedule = %obj.schedule(50, "liquidDamage", %data, %damageAmount, %damageType);
      passengerLiquidDamage(%obj, %damageAmount, %damageType);
   }
   else
      %obj.lDamageSchedule = "";
}

function HoverVehicle::liquidDamage(%obj, %data, %damageAmount, %damageType)
{
   if(%obj.getDamageState() !$= "Destroyed")
   {
      %data.damageObject(%obj, 0, "0 0 0", %damageAmount, %damageType);
      %obj.lDamageSchedule = %obj.schedule(50, "liquidDamage", %data, %damageAmount, %damageType);
      passengerLiquidDamage(%obj, %damageAmount, %damageType);
   }
   else
      %obj.lDamageSchedule = "";
}

function passengerLiquidDamage(%obj, %damageAmount, %damageType)
{
   for(%i = %num; %i < %obj.getDataBlock().numMountPoints; %i++)
      if (%p = %obj.getMountNodeObject(%i))
         %p.liquidDamage(%p.getDatablock(), $DamageLava, $DamageType::Lava);
}

function VehicleData::onLeaveLiquid(%data, %obj, %type)
{
   switch(%type)
   {
      case 0:
         //Water
         %obj.setHeat(1.0);
      case 1:
         //Ocean Water
         %obj.setHeat(1.0);
      case 2:
         //River Water
         %obj.setHeat(1.0);
      case 3:
         //Stagnant Water
         %obj.setHeat(1.0);
      case 4:
         //Lava
      case 5:
         //Hot Lava
      case 6:
         //Crusty Lava
      case 7:
         //Quick Sand
   }

   if(%obj.lDamageSchedule !$= "")
   {
      cancel(%obj.lDamageSchedule);
      %obj.lDamageSchedule = "";
   }
}

function VehicleData::onDestroyed(%data, %obj, %prevState)
{
	%destroyer = %obj.lastDamagedBy;
	game.vehicleDestroyed(%obj, %destroyer, %obj.lastDamageType);

	radiusVehicleExplosion(%data, %obj);
   	if(%obj.turretObject)
      	if(%obj.turretObject.getControllingClient())
         	%obj.turretObject.getDataBlock().playerDismount(%obj.turretObject);
   	for(%i = 0; %i < %obj.getDatablock().numMountPoints; %i++)
   	{
      	if (%obj.getMountNodeObject(%i))
      	{
         	%flingee = %obj.getMountNodeObject(%i);
         	%flingee.getDataBlock().doDismount(%flingee, true);
         	%xVel = 250.0 - (getRandom() * 500.0);
         	%yVel = 250.0 - (getRandom() * 500.0);
         	%zVel = (getRandom() * 100.0) + 50.0;
         	%flingVel = %xVel @ " " @ %yVel @ " " @ %zVel;
         	%flingee.applyImpulse(%flingee.getTransform(), %flingVel);
         	%flingee.damage(0, %obj.getPosition(), 0.4, $DamageType::Explosion);
      	}
   	}

   	%obj.schedule(2000, "delete");
}

function radiusVehicleExplosion(%data, %vehicle)
{
	// this is a modified version of RadiusExplosion() from projectiles.cs
	%position = %vehicle.getPosition();
   InitContainerRadiusSearch(%position, %data.explosionRadius, $TypeMasks::PlayerObjectType      |
                                                 $TypeMasks::VehicleObjectType     |
                                                 $TypeMasks::MoveableObjectType    |
                                                 $TypeMasks::StaticShapeObjectType |
                                                 $TypeMasks::ForceFieldObjectType  |
                                                 $TypeMasks::TurretObjectType      |
                                                 $TypeMasks::ItemObjectType);

   %numTargets = 0;
   while ((%targetObject = containerSearchNext()) != 0)
   {
		if(%targetObject == %vehicle)
			continue;

      %dist = containerSearchCurrRadDamageDist();

      if (%dist > %data.explosionRadius)
         continue;

      if (%targetObject.isMounted())
      {
         %mount = %targetObject.getObjectMount();
			if(%mount == %vehicle)
				continue;

         %found = -1;
         for (%i = 0; %i < %mount.getDataBlock().numMountPoints; %i++)
         {
            if (%mount.getMountNodeObject(%i) == %targetObject)
            {
               %found = %i;
               break;
            }
         }

         if (%found != -1)
         {
            if (%mount.getDataBlock().isProtectedMountPoint[%found] && (%mount != %vehicle))
               continue;
         }
      }

      %targets[%numTargets]     = %targetObject;
      %targetDists[%numTargets] = %dist;
      %numTargets++;
   }

   for (%i = 0; %i < %numTargets; %i++)
   {
      %targetObject = %targets[%i];
      %dist = %targetDists[%i];

      %coverage = calcExplosionCoverage(%position, %targetObject,
                                        ($TypeMasks::InteriorObjectType |
                                         $TypeMasks::TerrainObjectType |
                                         $TypeMasks::ForceFieldObjectType));
      if (%coverage == 0)
         continue;

      %amount = (1.0 - (%dist / %data.explosionRadius)) * %coverage * %data.explosionDamage;
      %targetData = %targetObject.getDataBlock();

      %momVec = "0 0 1";

      if(%amount > 0)
         %targetData.damageObject(%targetObject, %sourceObject, %position, %amount, $DamageType::Explosion, %momVec);
   }
}

function VehicleData::deleteAllMounted()
{

}


//**************************************************************
//* VEHICLE INVENTORY MANAGEMENT
//**************************************************************

//--------------------------------------------------------------
// NUMBER OF PURCHASEABLE VEHICLES PER TEAM
//--------------------------------------------------------------

$VehicleRespawnTime        = 15000;
$Vehiclemax[ScoutVehicle]     = 1;
$VehicleMax[TACAssaultVehicle]   = 2;
$VehicleMax[TACMobileBaseVehicle]  = 1;
$VehicleMax[TACScoutFlyer]       = 1;
$VehicleMax[TACBomberFlyer]      = 2;
$VehicleMax[TACFLPCFlyer]        = 4;
$VehicleMax[TACHAPCFlyer]        = 2;

function vehicleListRemove(%data, %obj)
{
   %blockName = %data.getName();
   for($i = 0; %i < $VehicleMax[%blockName]; %i++)
      if($VehicleInField[%obj.team, %blockName, %i] == %obj)
      {
         $VehicleInField[%obj.team, %blockName, %i] = 0;
         $VehicleTotalCount[%obj.team, %blockName]--;
         break;
      }
}

function vehicleListAdd(%blockName, %obj)
{
   for($i = 0; %i < $VehicleMax[%blockName]; %i++)
   {
      if($VehicleInField[%obj.team, %blockName, %i] $= "" || $VehicleInField[%obj.team, %blockName, %i] == 0)
      {
         $VehicleInField[%obj.team, %blockName, %i] = %obj;
         $VehicleTotalCount[%obj.team, %blockName]++;
         break;
      }
   }
}

function clearVehicleCount(%team)
{
   $VehicleTotalCount[%team, ScoutVehicle]      = 0;
   $VehicleTotalCount[%team, TACAssaultVehicle]    = 0;
   $VehicleTotalCount[%team, TACMobileBaseVehicle] = 0;
   $VehicleTotalCount[%team, TACScoutFlyer]        = 0;
   $VehicleTotalCount[%team, TACBomberFlyer]       = 0;
   $VehicleTotalCount[%team, TACFLPCFlyer]         = 0;
   $VehicleTotalCount[%team, TACHAPCFlyer]         = 0;
}

//**************************************************************
//* VEHICLE HUD SEAT INDICATOR LIGHTS
//**************************************************************

function findEmptySeat(%vehicle, %player, %forceNode)
{
   %minNode = 1;
   %node = -1;
   %dataBlock = %vehicle.getDataBlock();
   %dis = %dataBlock.minMountDist;
   %playerPos = getWords(%player.getTransform(), 0, 2);
   %message = "";
   if(%dataBlock.lightOnly)
   {
      if(%player.client.armor $= "Light")
         %minNode = 0;
      else
         %message = '\c2Only Scout Armors can pilot this vehicle.~wfx/misc/misc.error.wav';
   }
   else if(%player.client.armor $= "Light" || %player.client.armor $= "Medium")
      %minNode = 0;
   else
      %minNode = findFirstHeavyNode(%dataBlock);

   if(%forceNode !$= "")
      %node = %forceNode;
   else
   {
      for(%i = 0; %i < %dataBlock.numMountPoints; %i++)
         if(!%vehicle.getMountNodeObject(%i))
         {
            %seatPos = getWords(%vehicle.getSlotTransform(%i), 0, 2);
            %disTemp = VectorLen(VectorSub(%seatPos, %playerPos));
            if(%disTemp <= %dis)
            {
               %node = %i;
               %dis = %disTemp;
            }
         }
    }
   if(%node != -1 && %node < %minNode)
   {
      if(%message $= "")
      {
         if(%node == 0)
            %message = '\c2Only Scout or Assault Armors can pilot this vehicle.~wfx/misc/misc.error.wav';
         else
            %message = '\c2Only Scout or Assault Armors can use that position.~wfx/misc/misc.error.wav';
      }

      if(!%player.noSitMessage)
      {
         %player.noSitMessage = true;
         %player.schedule(2000, "resetSitMessage");
         messageClient(%player.client, 'MsgArmorCantMountVehicle', %message);
      }
      %node = -1;
   }
   return %node;
}



function findFirstHeavyNode(%data)
{
   for(%i = 0; %i < %data.numMountPoints; %i++)
      if(%data.mountPose[%i] $= "")
         return %i;
   return %data.numMountPoints;
}


//**************************************************************
//* DAMAGE FUNCTIONS
//**************************************************************

function VehicleData::damageObject(%data, %targetObject, %sourceObject, %position, %amount, %damageType, %momVec, %theClient, %proj)
{
   if(%proj !$= "")
   {
      if(%amount > 0 && %targetObject.lastDamageProj !$= %proj)
      {
         %targetObject.lastDamageProj = %proj;
         %targetObject.lastDamageAmount = %amount;
      }
      else if(%targetObject.lastDamageAmount < %amount)
         %amount = %amount - %targetObject.lastDamageAmount;
      else
         return;
   }

   // check for team damage
   %sourceClient = %sourceObject ? %sourceObject.getOwnerClient() : 0;
   %targetTeam = getTargetSensorGroup(%targetObject.getTarget());

   if(%sourceClient)
      %sourceTeam = %sourceClient.getSensorGroup();
   else if(isObject(%sourceObject) && %sourceObject.getClassName() $= "Turret")
      %sourceTeam = getTargetSensorGroup(%sourceObject.getTarget());
   else
      %sourceTeam = %sourceObject ? getTargetSensorGroup(%sourceObject.getTarget()) : -1;

    if(%sourceObject)
    {
        %targetObject.lastDamagedBy = %sourceObject;
        %targetObject.lastDamageType = %damageType;
    }
    else
    {
        %targetObject.lastDamagedBy = 0;
        %targetObject.lastDamageType = %damageType;
	}


   // Scale damage type & include shield calculations...
   if (%data.isShielded)
      %amount = %data.checkShields(%targetObject, %position, %amount, %damageType);


   %damageScale = %data.damageScale[%damageType];
   if(%damageScale !$= "")
      %amount *= %damageScale;

   if(%amount != 0)
      %targetObject.applyDamage(%amount);

   if(%targetObject.getDamageState() $= "Destroyed" )
   {
      if( %momVec !$= "")
         %targetObject.setMomentumVector(%momVec);
   }
}

function VehicleData::onImpact(%data, %vehicleObject, %collidedObject, %vec, %vecLen)
{
	if(%vecLen > %data.minImpactSpeed)
	{

		if(isObject(%collidedObject))
			%className = %collidedObject.getDataBlock().className;
		else
			%className = "bla";

		//Dont let players just dismounting vehicle damage them
		if(collidedObject.dismounted == %vehicleObject)
			return;

		//Collision with another vehicle or player
		if(%className $= "WheeledVehicleData" || %className $= "FlyingVehicleData" || %className $= "HoverVehicleData" || %className $= "Armor")
		{
			%vehicleObject.lastDamagedBy = %collidedObject;
			%vehicleObject.lastDamageType = $DamageType::Impact;
			%data.damageObject(%vehicleObject, %collidedObject, VectorAdd(%vec, %vehicleObject.getPosition),
						 %vecLen * %data.speedDamageScale, $DamageType::Impact);
		}
		//Collision with anything else
		else
		{
			%vehicleObject.lastDamagedBy = 0;
			%vehicleObject.lastDamageType = $DamageType::Ground;
			%data.damageObject(%vehicleObject, 0, VectorAdd(%vec, %vehicleObject.getPosition),
					 %vecLen * %data.speedDamageScale, $DamageType::Ground);
		}

	}

	// associated "crash" sounds
	if(%vecLen > %vDataBlock.hardImpactSpeed)
		%vehicleObject.playAudio(0, %vDataBlock.hardImpactSound);
	else if(%vecLen > %vDataBlock.softImpactSpeed)
		%vehicleObject.playAudio(0, %vDataBlock.softImpactSound);
}

//**************************************************************
//* VEHICLE CLASS CREATION
//**************************************************************

function HoverVehicleData::create(%block, %team, %oldObj)
{
   if(%oldObj $= "")
   {
      %obj = new HoverVehicle()
      {
         dataBlock  = %block;
         respawn    = "0";
         teamBought = %team;
         team = %team;
      };
   }
   else
   {
      %obj = new HoverVehicle()
      {
         dataBlock  = %data;
         respawn    = "0";
         teamBought = %team;
         team = %team;
         mountable = %oldObj.mountable;
         disableMove = %oldObj.disableMove;
         resetPos = %oldObj.resetPos;
         respawnTime = %oldObj.respawnTime;
         marker = %oldObj;
      };
   }
   return(%obj);
}

function WheeledVehicleData::create(%data, %team, %oldObj)
{
   if(%oldObj $= "")
   {
      %obj = new WheeledVehicle()
      {
         dataBlock  = %data;
         respawn    = "0";
         teamBought = %team;
         team = %team;
      };
   }
   else
   {
      %obj = new WheeledVehicle()
      {
         dataBlock  = %data;
         respawn    = "0";
         teamBought = %team;
         team = %team;
         mountable = %oldObj.mountable;
         disableMove = %oldObj.disableMove;
         resetPos = %oldObj.resetPos;
         deployed = %oldObj.deployed;
         respawnTime = %oldObj.respawnTime;
         marker = %oldObj;
      };
   }

   return(%obj);
}

function FlyingVehicleData::create(%data, %team, %oldObj)
{
   if(%oldObj $= "")
   {
      %obj = new FlyingVehicle()
      {
         dataBlock  = %data;
         respawn    = "0";
         teamBought = %team;
         team = %team;
      };
   }
   else
   {
      %obj = new FlyingVehicle()
      {
         dataBlock  = %data;
         teamBought = %team;
         team = %team;
         mountable = %oldObj.mountable;
         disableMove = %oldObj.disableMove;
         resetPos = %oldObj.resetPos;
         respawnTime = %oldObj.respawnTime;
         marker = %oldObj;
      };
   }

   return(%obj);
}

function FlyingVehicleData::switchSidesSetPos(%data, %oldObj)
{
   %team = %oldObj.curTeam == 1 ? 2 : 1;
   %oldObj.curTeam = %team;
   %obj = new FlyingVehicle()
   {
      dataBlock  = %data;
      teamBought = %team;
      team = %team;
      mountable = %oldObj.mountable;
      disableMove = %oldObj.disableMove;
      resetPos = %oldObj.resetPos;
      respawnTime = %oldObj.respawnTime;
      marker = %oldObj;
   };
   %obj.setTransform(%oldObj.getTransform());

   return(%obj);
}

function WheeledVehicleData::switchSidesSetPos(%data, %oldObj)
{
   %team = %oldObj.curTeam == 1 ? 2 : 1;
   %oldObj.curTeam = %team;
   %obj = new WheeledVehicle()
   {
      dataBlock  = %data;
      respawn    = "0";
      teamBought = %team;
      team = %team;
      mountable = %oldObj.mountable;
      disableMove = %oldObj.disableMove;
      resetPos = %oldObj.resetPos;
      deployed = %oldObj.deployed;
      respawnTime = %oldObj.respawnTime;
      marker = %oldObj;
   };
   %obj.setTransform(%oldObj.getTransform());
   return(%obj);
}

function HoverVehicleData::switchSides(%data, %oldObj)
{
   %team = %oldObj.curTeam == 1 ? 2 : 1;
   %oldObj.curTeam = %team;
   %obj = new HoverVehicle()
   {
      dataBlock  = %data;
      respawn    = "0";
      teamBought = %team;
      team = %team;
      mountable = %oldObj.mountable;
      disableMove = %oldObj.disableMove;
      resetPos = %oldObj.resetPos;
      respawnTime = %oldObj.respawnTime;
      marker = %oldObj;
   };
   %obj.setTransform(%oldObj.getTransform());
   return(%obj);
}

function resetNonStaticObjPositions()
{
   MissionGroup.setupPositionMarkers(false);
   MissionCleanup.positionReset();
}

function next(%team)
{
   ResetObjsPositions(%team);
}

function SimGroup::positionReset(%group)
{
   for(%i = %group.getCount() - 1; %i >=0 ; %i--)
   {
      %obj = %group.getObject(%i);
      if(%obj.resetPos && %obj.getName() !$= PosMarker)
         %obj.delete();
      else
         %obj.positionReset();
   }

   for(%i = 0; %i < %group.getCount(); %i++)
   {
      %obj = %group.getObject(%i);
      if(%obj.getName() $= PosMarker)
      {
         cancel(%obj.schedule);
         %newObj =  %obj.data.switchSidesSetPos(%obj);
         MissionCleanup.add(%newObj);
         setTargetSensorGroup(%newObj.target, %newObj.team);
      }
      else
         %obj.positionReset();
   }
}

function VehicleData::respawn(%data, %marker)
{
   %mask = $TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType | $TypeMasks::TurretObjectType;
   InitContainerRadiusSearch(%marker.getWorldBoxCenter(), %data.checkRadius, %mask);
   if(containerSearchNext() == 0)
   {
      %newObj = %data.create(%marker.curTeam, %marker);
      %newObj.startFade(1000, 0, false);
      %newObj.setTransform(%marker.getTransform());

      setTargetSensorGroup(%newObj.target, %newObj.team);
      MissionCleanup.add(%newObj);
   }
   else
   {
      %marker.schedule = %data.schedule(3000, "respawn", %marker);
   }
}

function SimObject::positionReset(%group, %team)
{
   //Used to avoid warnings
}

function Terraformer::positionReset(%group, %team)
{
   //Used to avoid warnings
}

function SimGroup::setupPositionMarkers(%group, %create)
{
   for(%i = %group.getCount() - 1; %i >= 0; %i--)
   {
      %obj = %group.getObject(%i);
      if(%obj.resetPos || %obj.respawnTime !$= "")
      {
         if(%create)
         {
            %marker = %obj.getDataBlock().createPositionMarker(%obj);
            MissionCleanup.add(%marker);
            %obj.marker = %marker;
         }
         else
         {
            %obj.delete();
         }
      }
      else
         %obj.setupPositionMarkers(%create);
   }
}

function SimObject::setupPositionMarkers(%group, %create)
{
   //Used to avoid warnings
}

function VehicleData::createPositionMarker(%data, %obj)
{
   %marker = new Trigger(PosMarker)
   {
      dataBlock = markerTrigger;
      mountable = %obj.mountable;
      disableMove = %obj.disableMove;
      resetPos = %obj.resetPos;
      data = %obj.getDataBlock().getName();
      deployed = %obj.deployed;
      curTeam = %obj.team;
      respawnTime = %obj.respawnTime;
   };
   %marker.setTransform(%obj.getTransform());
   return %marker;
}

function VehicleData::hasDismountOverrides(%data, %obj)
{
   return false;
}


//=============================================================
//	Pilot Fire Code

function ShapeBase::FixedRepairVehicle(%this, %TotalRepairDamage, %RepairTime, %RepairEnergy)
{
	//This function can be used to allow the pilot to repair any vehicle.
	//%TotalRepairDamage is the health added over period
	//%RepairTime is the time for repair to complete x 100ms
	//%RepairEnergy is the energy sapped on use

	%energy = %this.getEnergyLevel();
	if(%energy < %RepairEnergy)
		return;

	%damage = %this.getDamageLevel();
	if(%damage == 0)
		return;

	%this.setEnergyLevel(%energy - %RepairEnergy);
	%repairdamage = %TotalRepairDamage / %RepairTime;
	%this.timeBasedRepair(%repairdamage, %RepairTime);
}

function ShapeBase::timeBasedRepair(%this, %repairdamage, %repairremaining)
{
	%damage = %this.getDamageLevel();
	if(%damage == 0)
	{
		%this.stopAudio(0);
		return;
	}

	if(%damage < %repairdamage)
		%repairdamage = %damage;

	if(%repairremaining > 0)
	{
		%this.playAudio(0, RepairPackFireSound);
		%this.setDamageLevel(%damage - %repairdamage);
		%repairremaining = %repairremaining - 1;
		%this.lastrepair = %this.schedule(100, timeBasedRepair, %repairdamage, %repairremaining);
	}
	else
	{
		%this.stopAudio(0);
	}
}


function ShapeBase::RepairVehicle(%this, %state, %RepairRate, %RepairEnergy)
{
	//This function can be used to allow the pilot to repair any vehicle.
	//%RepairRate is the health added per 100ms
	//%RepairEnergy is the energy sapped by use per 100ms
	//%state can be true to activate and false to deactivate

	%energy = %this.getEnergyLevel();
	%damage = %this.getDamageLevel();
	if(!%state || %energy<=%RepairEnergy || %damage <= 0)
	{
		%this.RepairActive = false;
		%this.RepairCheck = false;
		%this.stopAudio(0);
		cancel(%this.lastrepair);
		%this.lastrepair = 0;
	}
	else
	{
		if(%this.RepairCheck)
		{
			%this.setDamageLevel(%damage - %RepairRate);
			%this.setEnergyLevel(%energy - %RepairEnergy);
			%this.playAudio(0, RepairPackFireSound);
			%this.lastrepair = %this.schedule(100, RepairVehicle, true, %RepairRate, %RepairEnergy);
		}
	}
}

function ShapeBase::CloakVehicle(%this, %state, %CloakEnergy, %pilotid, %turret, %pass1, %pass2, %pass3, %pass4, %pass5)
{
	//This function can be used to allow the pilot to cloak the vehicle.
	//%CloakEnergy is the energy sapped by use per 100ms
	//%state can be true to activate and false to deactivate

	%energy = %this.getEnergyLevel();
	if(!%state || %energy<=%CloakEnergy)
	{
		%this.setCloaked(false);
		%decal = Game.DefineTeamSkin(%this.team, "vehicle");
		%this.schedule(100, "playThread", $ActivateThread, "activate");
		if(%decal !$= "")
			%this.schedule(100, "playThread", $VehicleDecalThread, %decal);
		if(%turret != 0)
			%turret.setCloaked(false);
		if(%pilotid != 0)
			%pilotid.setCloaked(false);
		if(%pass1 != 0)
			%pass1.setCloaked(false);
		if(%pass2 != 0)
			%pass2.setCloaked(false);
		if(%pass3 != 0)
			%pass3.setCloaked(false);
		if(%pass4 != 0)
			%pass4.setCloaked(false);
		if(%pass5 != 0)
			%pass5.setCloaked(false);
		%this.CloakActive = false;
		%this.CloakCheck = false;
		%this.stopAudio(0);
		messageClient(%this.lastPilot, 'MsgVehicleCloakOff', 'Vehicle Cloak Deactivated.');
		cancel(%this.lastcloak);
		%this.lastcloak = 0;
	}
	else
	{
		if(%this.CloakCheck)
		{
			%this.schedule(100, "playThread", $ActivateThread, "invis");
			%this.schedule(100, "playThread", $VehicleDecalThread, "invis");
			%this.setCloaked(true);
			if(%turret != 0)
				%turret.setCloaked(true);
			if(%pilotid != 0)
				%pilotid.setCloaked(true);
			if(%pass1 != 0)
				%pass1.setCloaked(true);
			if(%pass2 != 0)
				%pass2.setCloaked(true);
			if(%pass3 != 0)
				%pass3.setCloaked(true);
			if(%pass4 != 0)
				%pass4.setCloaked(true);
			if(%pass5 != 0)
				%pass5.setCloaked(true);
			%this.setEnergyLevel(%energy - %CloakEnergy);
			%this.playAudio(0, CloakingPackActivateSound);
			messageClient(%this.lastPilot, 'MsgVehicleCloakOn', 'Vehicle Cloak Activated.');
			%this.lastcloak = %this.schedule(100, CloakVehicle, true, %CloakEnergy, %pilotid, %turret, %pass1, %pass2, %pass3, %pass4, %pass5);
		}
	}
}


function ShapeBase::ShieldVehicle(%this, %state, %ShieldEnergy)
{
	//This function can be used to allow the pilot to shield the vehicle.
	//%ShieldEnergy is the energy sapped by passive use per 100ms
	//%state can be true to activate and false to deactivate

	%energy = %this.getEnergyLevel();
	if(!%state || %energy<=%ShieldEnergy)
	{
		%this.isShielded = false;
		%this.ShieldActive = false;
		%this.ShieldCheck = false;
		%this.stopAudio(0);
		messageClient(%this.lastPilot, 'MsgVehicleShieldOff', 'Vehicle Shield Deactivated.');
		cancel(%this.lastshield);
		%this.lastshield = 0;
	}
	else
	{
		if(%this.ShieldCheck)
		{
			%this.isShielded = true;
			%this.setEnergyLevel(%energy - %ShieldEnergy);
			%this.playAudio(0, ShieldPackActivateSound);
			messageClient(%this.lastPilot, 'MsgVehicleShieldOn', 'Vehicle Shield Activated.');
			%this.lastshield = %this.schedule(100, ShieldVehicle, true,%ShieldEnergy);
		}
	}
}

function ShapeBase::UseVehicleFlareGrenade(%this, %FlareEnergyCost)
{
	%energy = %this.getEnergyLevel();
	if(%energy > %FlareEnergyCost)
	{
		if((%this.NextFlareFire - getSimTime() > 0) && (%this.NextFlareFire > 0))
			return;
		%this.setEnergyLevel(%energy - %FlareEnergyCost);
		%p = new FlareProjectile()
		{
			dataBlock        = FlareGrenadeProj;
			initialDirection = "0 0 0.4";
			initialPosition  = getBoxCenter(%this.getWorldBox());
			sourceObject     = %this;
			sourceSlot       = 0;
		};
		%this.playAudio(0, GrenadeFireSound);
		FlareSet.add(%p);
		%this.NextFlareFire = getSimTime() + 3000;
		MissionCleanup.add(%p);
		%p.schedule(6000, "delete");
	}
	else
		%this.playAudio(0, GrenadeDryFireSound);
}


//**************************************************************
//* VEHICLE TIMEOUT
//**************************************************************

$VehicleTimeOut = 120000;

function VehicleData::CheckVehicleMounted(%data, %obj)
{
	for(%mount=0; %mount <= %obj.getDatablock().numMountPoints; %mount++)
	{
		if (%obj.getMountNodeObject(%mount))
			return true;
	}
	return false;
}

function VehicleData::TimeoutVehicle(%data, %vehicle)
{
	if(!isObject(%vehicle))
		return;

	if(%data.CheckVehicleMounted(%vehicle))
		return;

	if(%vehicle.respawnTime !$= "")
		%vehicle.marker.schedule = %vehicle.marker.data.schedule(%vehicle.respawnTime, "respawn", %vehicle.marker);
	%vehicle.mountable = 0;
	%vehicle.startFade(1000, 0, true);
	%vehicle.schedule(1001, "delete");
}

function VehicleData::playerDismounted(%data, %obj, %player)
{
	if(%data.cantAbandon)
		return;
	%player.dismounted = %obj;
	%player.schedule(200,"ResetPlayerMount");

	if(!%data.CheckVehicleMounted(%obj))
	{
		if(%data.lastdismount)
		{
			cancel(%data.lastdismount);
			%data.lastdismount = 0;
		}
		%data.lastdismount = %data.schedule($VehicleTimeOut,"TimeoutVehicle",%obj, %obj.lastdismounttime);
	}

   	if( %player.client.observeCount > 0 )
      	resetObserveFollow( %player.client, true );
}


